/******************************************************************************
*   TinTin++                                                                  *
*   Copyright (C) 2005 (See CREDITS file)                                     *
*                                                                             *
*   This program is protected under the GNU GPL (See COPYING)                 *
*                                                                             *
*   This program is free software; you can redistribute it and/or modify      *
*   it under the terms of the GNU General Public License as published by      *
*   the Free Software Foundation; either version 2 of the License, or         *
*   (at your option) any later version.                                       *
*                                                                             *
*   This program is distributed in the hope that it will be useful,           *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of            *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
*   GNU General Public License for more details.                              *
*                                                                             *
*   You should have received a copy of the GNU General Public License         *
*   along with this program; if not, write to the Free Software               *
*   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
*******************************************************************************/

/******************************************************************************
*   file: list.c - pseudo array support                                       *
*                                                                             *
*               (T)he K(I)cki(N) (T)ickin D(I)kumud Clie(N)t                  *
*                                                                             *
*                     coded by Igor van den Hoven 2004                        *
******************************************************************************/


#include "tintin.h"


DO_COMMAND(do_list)
{
	char arg1[BUFFER_SIZE], arg2[BUFFER_SIZE];
	struct listroot *root;
	struct listnode *node;
	int cnt;

	root = ses->list[LIST_VARIABLE];

	arg = sub_arg_in_braces(ses, arg, arg1, GET_NST, SUB_VAR|SUB_FUN);
	arg = sub_arg_in_braces(ses, arg, arg2, GET_ONE, SUB_VAR|SUB_FUN);

	if (*arg1 == 0 || *arg2 == 0)
	{
		show_message(ses, LIST_VARIABLE, "#SYNTAX: #LIST {listname} {CLR|DEL|FND|GET|INS|LEN|SET|SRT} {argument}.");
	}
	else
	{
		for (cnt = 0 ; *array_table[cnt].name ; cnt++)
		{
			if (is_abbrev(arg2, array_table[cnt].name))
			{
				break;
			}
		}

		if (*array_table[cnt].name == 0)
		{
			show_message(ses, LIST_VARIABLE, "#SYNTAX: #LIST {%s} {CLR|DEL|FND|INS|GET|SET|LEN} {argument}.", arg1);
		}
		else
		{
			if ((node = search_nest_node(root, arg1)) == NULL)
			{
				node = set_nest_node(root, arg1, "");
			}
			arg = sub_arg_in_braces(ses, arg, arg1, array_table[cnt].lval, SUB_VAR|SUB_FUN);
			arg = sub_arg_in_braces(ses, arg, arg2, array_table[cnt].rval, SUB_VAR|SUB_FUN);

			array_table[cnt].array(ses, node, arg1, arg2);
		}
	}
	return ses;
}

DO_ARRAY(array_clr)
{
	if (list->root)
	{
		free_list(list->root);

		list->root = NULL;
	}
	set_nest_node(ses->list[LIST_VARIABLE], list->left, "");

	return ses;
}

DO_ARRAY(array_del)
{
	int index, cnt;

	if (list->root)
	{
		index = search_nest_index(list->root, left);

		if (atoi(left) == 0 || index == -1)
		{
			tintin_printf2(ses, "#LIST DEL: Invalid index: %s", left);

			return ses;
		}

		for (cnt = index + 1 ; cnt < list->root->used ; cnt++)
		{
			list->root->list[cnt]->left = refstring(list->root->list[cnt]->left, "%d", cnt);
		}

		delete_index_list(list->root, index);

		return ses;
	}

	show_message(ses, LIST_VARIABLE, "#LIST DEL: {%s} is not a list.", list->left);

	return ses;
}

DO_ARRAY(array_fnd)
{
	int cnt, index;

	if (*right == 0)
	{
		show_message(ses, LIST_VARIABLE, "#SYNTAX: #LIST FND {string} {variable}");
		return ses;
	}

	if (list->root)
	{
		for (cnt = 0 ; cnt < list->root->used ; cnt++)
		{
			if (match(ses, list->root->list[cnt]->right, left))
			{
				index = cnt;
				break;
			}
		}
		if (cnt < list->root->used)
		{
			set_nest_node(ses->list[LIST_VARIABLE], right, "%d", index + 1);
		}
		else
		{
			set_nest_node(ses->list[LIST_VARIABLE], right, "0");
		}
		return ses;
	}
	else
	{
		set_nest_node(ses->list[LIST_VARIABLE], right, "0");
	}

	return ses;
}

DO_ARRAY(array_get)
{
	if (*right == 0)
	{
		show_message(ses, LIST_VARIABLE, "#SYNTAX: #LIST GET {index} {variable}");
		return ses;
	}

	if (list->root)
	{
		int index = search_nest_index(list->root, left);

		if (atoi(left) == 0 || index == -1)
		{
			set_nest_node(ses->list[LIST_VARIABLE], right, "0");
		}
		else
		{
			set_nest_node(ses->list[LIST_VARIABLE], right, "%s", list->root->list[index]->right);
		}
		return ses;
	}
	else
	{
		set_nest_node(ses->list[LIST_VARIABLE], right, "0");
	}

	return ses;
}

DO_ARRAY(array_ins)
{
	char temp[BUFFER_SIZE];
	int cnt, index;

	if (!list->root)
	{
		list->root = init_list(ses, LIST_VARIABLE, LIST_SIZE);
	}

	if (list->root)
	{
		index = search_nest_index(list->root, left);

		if (atoi(left) == 0)
		{
			tintin_printf2(ses, "#LIST INS: Invalid index: %s", left);

			return ses;
		}

		if (index == -1 || atoi(left) < 0)
		{
			index++;
		}

		for (cnt = index ; cnt < list->root->used ; cnt++)
		{
			list->root->list[cnt]->left = refstring(list->root->list[cnt]->left, "%d", cnt + 2);
		}

		sprintf(temp, "%d", index + 1);

		insert_node_list(list->root, temp, right, "");

		return ses;
	}

	show_message(ses, LIST_VARIABLE, "#ERROR: LIST INS: {%s} is not a list.", list->left);

	return ses;
}

DO_ARRAY(array_len)
{
	if (*left == 0)
	{
		show_message(ses, LIST_VARIABLE, "#SYNTAX: #LIST GET {index} {variable}");
		return ses;
	}

	if (list->root)
	{
		set_nest_node(ses->list[LIST_VARIABLE], left, "%d", list->root->used);

		return ses;
	}
	set_nest_node(ses->list[LIST_VARIABLE], left, "0");

	return ses;
}

DO_ARRAY(array_set)
{
	if (list->root)
	{
		int index = search_nest_index(list->root, left);

		if (atoi(left) == 0 || index == -1)
		{
			tintin_printf2(ses, "#LIST SET: Invalid index: %s", left);

			return ses;
		}

		RESTRING(list->root->list[index]->right, right);

		return ses;
	}

	show_message(ses, LIST_VARIABLE, "#LIST SET: {%s} is not a list.", list->left);

	return ses;
}

DO_ARRAY(array_srt)
{
	char temp[BUFFER_SIZE];
	int cnt;

	if (!list->root)
	{
		list->root = init_list(ses, LIST_VARIABLE, LIST_SIZE);
	}

	if (list->root)
	{
		for (cnt = 0 ; cnt < list->root->used ; cnt++)
		{
			if (strcmp(left, list->root->list[cnt]->right) <= 0)
			{
				break;
			}
		}

		sprintf(temp, "%d", cnt + 1);

		if (cnt == list->root->used)
		{
			insert_node_list(list->root, temp, left, "");
		}
		else
		{
			array_ins(ses, list, temp, left);
		}
		return ses;
	}

	show_message(ses, LIST_VARIABLE, "#LIST SET: {%s} is not a list.", list->left);

	return ses;
}
